#!/usr/bin/env python
# Deprecated: packaging is now configured in pyproject.toml.
# This file is kept for backward compatibility with old tooling.

from setuptools import setup

if __name__ == "__main__":
    setup()
